library ieee;
use ieee.std_logic_1164.all;

entity i2c_temperatura is
port(
	clk_i 			: in std_logic;
   rst_i 			: in std_logic;
	start_write_i	: in std_logic;
	i2c_address_i	: in std_logic_vector (7 downto 0);
	i2c_write_i		: in std_logic_vector (15 downto 0);
	i2c_sda_io		: inout std_logic;
	i2c_scl_o		: out std_logic;
	i2c_busy_o		: out std_logic;
	i2c_read_o		: out std_logic_vector (11 downto 0)
);
end i2c_temperatura;

architecture behavior of i2c_temperatura is

-- Definiçao as constantes
	constant RD       : std_logic := '1';
	constant WT       : std_logic := '0';
	constant TEMP_REG : std_logic_vector(3 downto 0) := "0000"; 
	constant CONF_REG : std_logic_vector(3 downto 0) := "0001"; 
	constant TLOW_REG : std_logic_vector(3 downto 0) := "0010";
	constant THIGH_REG : std_logic_vector(3 downto 0) := "0011";

-- Definiçao dos sinais
	signal rw, sda, dir_sda	: std_logic := '0';
	signal i2c_read_sig		: std_logic := '0';
	signal cs_end, cs			: std_logic := '1';
	signal tmp_register		: std_logic_vector (1 downto 0) := "00";
	signal counter				: integer range 0 to 64 := 0;
	signal counter_length	: integer range 0 to 64 := 0;
	signal buffer_lenght		: integer range 0 to 64 := 0;
	signal data_read			: std_logic_vector (11 downto 0) := (others => '0');
	signal buffer_read		: std_logic_vector (15 downto 0) := (others => '0');
	signal buffer_write		: std_logic_vector (35 downto 0) := (others => '0');
	signal sda_in				: std_logic;
	signal i2c_busy_sig		: std_logic;
   
   signal clk_div_posedge       : std_logic := '0';
   signal clk_div_negedge  : std_logic := '0';
   signal clk_div          : std_logic := '0';
begin
	
   -- Divisor de clock
	process(clk_i, rst_i) 
   variable clk_div_cntr: integer range 0 to 125 := 0;	
   begin
   if (rst_i = '1') then
      clk_div_cntr := 0;
      clk_div_posedge <= '0';
      clk_div_negedge <= '0';
      clk_div <= '0';      
   elsif rising_edge (clk_i) then
      if (clk_div_cntr < 125) then clk_div_cntr := clk_div_cntr + 1;
      else clk_div_cntr := 0; 
      end if;
      if (clk_div_cntr > 125/2) then clk_div <= '1';
      else clk_div <= '0';
      end if;
      if (clk_div_cntr = 125/2 + 1) then clk_div_posedge <= '1';
      else clk_div_posedge <= '0';
      end if;
      if (clk_div_cntr = 0) then clk_div_negedge <= '1';
      else clk_div_negedge <= '0';
      end if;
   end if;
   end process;
	
	-- Processo de controle
	Controle:
	process(clk_i)
	begin
	if rising_edge(clk_i) then
      if (clk_div_posedge='1') then
         cs <= '1';
         if (start_write_i = '1' and i2c_busy_sig = '0') then
            cs <= '0';
            counter <= 0;
            buffer_write <= i2c_address_i & '0' & "000000" & i2c_write_i(1 downto 0) & '0' & i2c_write_i(15 downto 8) & '0' & i2c_write_i(7 downto 4) & "0000" & '0';
            rw <= i2c_address_i(0);
            tmp_register <= i2c_write_i(1 downto 0);
            -- Definiçao do tamanho do dado
            counter_length <= 36;
            buffer_lenght <= 36;
            if (i2c_address_i(0) = RD) then
               counter_length <= 27;
               buffer_lenght <= 8;
               if (i2c_write_i(1 downto 0) = CONF_REG(1 downto 0)) then
                  counter_length <= 18;
               end if;
            elsif (i2c_write_i(1 downto 0) = CONF_REG (1 downto 0)) then
               counter_length <= 27;
               buffer_lenght <= 27;
            elsif (i2c_write_i(1 downto 0) = TEMP_REG(1 downto 0)) then
               counter_length <= 18;
               buffer_lenght <= 18;
            end if;
         elsif (cs = '0') then
            cs <= '0';
            buffer_write <= buffer_write(34 downto 0)  & '0';
            if (counter = counter_length) then
               cs <= '1';
            else	
               counter <= counter + 1;
            end if;
         end if;
         
         -- Leitura de dados da I2C
         if (counter > 8 and rw = '1' and cs = '0') then
            if (counter = 16 and tmp_register = "01") then
               data_read <= "0000" & buffer_read(6 downto 0) & i2c_sda_io;
            elsif (counter = 25) then
               data_read <= buffer_read(15 downto 8) & buffer_read (6 downto 3);
            else 
               buffer_read <= buffer_read(14 downto 0) & sda_in;
            end if;
         end if;
      end if;
	end if;
end process;

	-- Escrita de dado pela I2C
	Escrita:
	process (clk_i)
	begin
	if rising_edge(clk_i) then
      if (clk_div_negedge = '1') then
         cs_end <= '1';
         if (cs = '0') then
            cs_end <= '0';
            if (counter <= buffer_lenght) then
               sda <= buffer_write(35);
               dir_sda <= '0';
            elsif (counter = counter_length or (rw = '1' and counter = 17)) then
               sda <= '0';
               dir_sda <= '0';
            else
               dir_sda <= '1';
            end if;
            if (counter = 8 or (rw = '0' and counter = 17) or counter = 26 or counter = 35) then
               dir_sda <= '1';
            end if;
         end if;
      end if; 
	end if;
	end process;

-- Clock passado para o sensor, utilizando o pull-up da saida
i2c_scl_o <= 'H' when clk_div = '1' or cs_end = '1' or cs = '1' else
				 '0';
i2c_busy_sig <= ((not cs) or (not cs_end));
i2c_busy_o <= i2c_busy_sig;
-- Definiçao da saida, considerando o pull-up no sinal i2c_sda_io
i2c_sda_io <= 	'H' when ((dir_sda = '0' or i2c_busy_sig = '0') or (clk_div = '1' and counter = 0)) and (sda = '1' or i2c_busy_sig = '0') else
					'0' when ((dir_sda = '0' or i2c_busy_sig = '0') or (clk_div = '1' and counter = 0)) and (sda = '0' or i2c_busy_sig = '1') else
					'Z';
i2c_read_o <= data_read;
sda_in <= i2c_sda_io;

end behavior;